package PendulumClock "Package for the Pendulum Clock modelica example."
  package Components "Components used in the pendulum clock example"
    extends Modelica.Icons.Package;

    model EscapementWheel "Model of clock escapement mechanism."
      //parameter Modelica.SIunits.Length depth(displayUnit = "cm") = 0.1;
      parameter Modelica.Units.SI.Length s0(displayUnit = "cm") = 0 "Relative position of cog end";
      parameter Modelica.Units.SI.Angle theta(displayUnit = "deg") = 45 "Cog angle";
      parameter Modelica.Units.SI.Length r(displayUnit = "cm") = 30 "Radius of wheel";
      Modelica.Mechanics.Translational.Interfaces.Flange_a impulseFace annotation(Placement(visible = true, transformation(origin = {-50, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {50, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Interfaces.Flange_a lockFace annotation(Placement(visible = true, transformation(origin = {-52.651, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-50, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b annotation(Placement(visible = true, transformation(origin = {50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Boolean lockContact, impulseContact;
      Modelica.Mechanics.Rotational.Interfaces.Flange_b rflange_b annotation(Placement(visible = true, transformation(origin = {-2.166, 0.676}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-0, -0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    equation
      impulseContact = if impulseFace.s > s0 and der(impulseFace.s) < 0 then true else false;
      lockContact = if lockFace.s > s0 then true else false;
      impulseFace.f = if impulseContact then flange_b.f * sin(theta) else 0;
      lockFace.f = 0;
      der(flange_b.s) = if lockContact then 0 else 0.01;
      rflange_b.phi = 2 * Modelica.Constants.pi * flange_b.s / r;
      annotation(Documentation(info = "<html><div>
<div>The escapement wheel serves two purposes. First, it locks the clock hand and gears in place while the escapement fork is in contact with the clock, and second, it pushes back on the fork when it discharges. This provides enough energy for the pendulum to compensate for the energy lost due to friction during the swing.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Polygon(visible = true, origin = {-58.895, 52.459}, rotation = -304.097, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Polygon(visible = true, origin = {-78.986, 2.97}, rotation = -270, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Polygon(visible = true, origin = {-55.841, -56.476}, rotation = -587.423, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Polygon(visible = true, origin = {-1.06, -78.986}, rotation = -180, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Polygon(visible = true, origin = {56.833, -54.607}, rotation = -484.097, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Polygon(visible = true, origin = {77.271, -2.97}, rotation = -90, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Polygon(visible = true, origin = {-1.902, 78.986}, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Polygon(visible = true, origin = {62.009, 46.801}, rotation = -407.423, fillColor = {58, 58, 60}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{6.774, -11.014}, {8.94, 11.014}, {-0.805, 11.014}, {-0.805, 4.476}, {-2.712, -1.559}, {-4.594, -4.685}, {-14.88, -11.014}}), Ellipse(visible = true, origin = {0, -1.904}, fillColor = {188, 189, 191}, fillPattern = FillPattern.Solid, lineThickness = 5, extent = {{-68.868, -71.904}, {68.868, 71.904}}), Ellipse(visible = true, origin = {0, -1.906}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 5, extent = {{-57.737, -60.282}, {57.737, 60.282}}), Rectangle(visible = true, origin = {-0.271, -1.127}, fillColor = {88, 88, 90}, fillPattern = FillPattern.Solid, extent = {{-5.143, -61.127}, {5.143, 61.127}}), Rectangle(visible = true, origin = {-0.271, 0}, rotation = -90, fillColor = {88, 88, 90}, fillPattern = FillPattern.Solid, extent = {{-5.143, -57.653}, {5.143, 57.653}})}), Diagram(coordinateSystem(extent = {{-52.032, -30}, {50, 30}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Text(visible = true, origin = {-1.658, 1.15}, extent = {{-46.658, -21.15}, {46.658, 21.15}}, textString = "Textual Model")}));
    end EscapementWheel;

    model EscapementFork "Model of escapement fork."
      Modelica.Mechanics.Translational.Components.Mass lock(m(displayUnit = "g") = 0, L.displayUnit = "cm", L = 0) annotation(Placement(visible = true, transformation(origin = {0, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass impulse(m(displayUnit = "g") = 0, L.displayUnit = "cm", L = 0, s.start = 0) annotation(Placement(visible = true, transformation(origin = {-0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.IdealGearR2T rightSide(ratio = 1) annotation(Placement(visible = true, transformation(origin = {-60, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.IdealGearR2T leftSide(ratio = -1) annotation(Placement(visible = true, transformation(origin = {-60, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a annotation(Placement(visible = true, transformation(origin = {-170, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, 1.82}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Interfaces.Flange_b lockFace annotation(Placement(visible = true, transformation(origin = {170, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {50, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Translational.Interfaces.Flange_b impulseFace annotation(Placement(visible = true, transformation(origin = {170, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-50, -102.402}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    equation
      connect(impulse.flange_b, impulseFace) annotation(Line(visible = true, origin = {90, -40}, points = {{-80, 0}, {80, 0}}, color = {0, 127, 0}));
      connect(lock.flange_b, lockFace) annotation(Line(visible = true, origin = {90, 40}, points = {{-80, 0}, {80, 0}}, color = {0, 127, 0}));
      connect(flange_a, leftSide.flangeR) annotation(Line(visible = true, origin = {-100, -20}, points = {{-70, 20}, {20, 20}, {20, -20}, {30, -20}}, color = {64, 64, 64}));
      connect(flange_a, rightSide.flangeR) annotation(Line(visible = true, origin = {-100, 20}, points = {{-70, -20}, {20, -20}, {20, 20}, {30, 20}}, color = {64, 64, 64}));
      connect(leftSide.flangeT, impulse.flange_a) annotation(Line(visible = true, origin = {-30, -40}, points = {{-20, 0}, {20, 0}}, color = {0, 127, 0}));
      connect(rightSide.flangeT, lock.flange_a) annotation(Line(visible = true, origin = {-30, 40}, points = {{-20, 0}, {20, 0}}, color = {0, 127, 0}));
      annotation(Documentation(info = "<html><div>
<div>The fork applies a braking force on a gear. When the pendulum pushes on the fork, and if the translational position of the pendulum exceeds a certain threshold, the fork will release its grip on the gear. When the pendulum swings back, the grip will be restored. The fork also transfers energy from the escapement wheel to the pendulum to compensate for energy lost due to friction.</div>
</div>
<div>
<div>
<p>The escapement fork here is modeled using two <a href=\"modelica://Modelica.Mechanics.Rotational.Components.IdealGearR2T\">rotational to translational</a> components with opposite signs on their ratios. When one face of the fork goes down, the other swings up.</p>
</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Polygon(visible = true, origin = {2.194, -35.46}, fillColor = {109, 110, 113}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{-63.699, -61.096}, {-64.871, -61.982}, {-82.194, -26.795}, {-73.699, 1.355}, {-69.743, 8.904}, {-55.668, 18.904}, {-36.721, 30.588}, {-23.699, 35.46}, {19.038, 35.46}, {43.399, 25.175}, {62.346, 11.1}, {73.714, -13.802}, {76.301, -24.088}, {66.301, -46.283}, {56.391, -61.441}, {56.301, -61.096}, {50.436, -23.547}, {46.301, 3.521}, {36.301, 12.724}, {28.782, 18.904}, {17.955, 18.904}, {-23.699, 20.844}, {-41.052, 17.054}, {-50.254, 8.904}, {-56.751, -11.096}}, smooth = Smooth.Bezier), Polygon(visible = true, origin = {0.598, 6.909}, fillColor = {189, 190, 192}, fillPattern = FillPattern.Solid, lineThickness = 5, points = {{-23.006, -21.526}, {-23.122, -22.004}, {-20.81, -3.465}, {-15.427, 6.535}, {-6.765, 12.579}, {-0.81, 16.535}, {9.19, 10.955}, {16.513, 3.376}, {19.19, -3.465}, {21.926, -21.526}, {21.501, -21.201}}, smooth = Smooth.Bezier)}), Diagram(coordinateSystem(extent = {{-170, -105}, {170, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
    end EscapementFork;

    model PeriodCounter "Component used for recording length of pendulum period."
      parameter Modelica.Units.SI.Time period_0 = 0 "Period initialization";
      Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a annotation(Placement(visible = true, transformation(origin = {-50, -0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Units.SI.Time t(start = 0) "Time initalization";
      discrete Modelica.Units.SI.Time period(start = period_0);
      discrete Modelica.Units.SI.Time lastperiod(start = period_0);
      Modelica.Units.SI.Time tdrift;
      Boolean count;
    equation
      flange_a.f = 0;
      count = if der(flange_a.s) > 0 then true else false;
      der(t) = if count then 1 else 0;
      tdrift = if period > 0 and lastperiod > 0 then period - lastperiod else 0;
    algorithm
      when not count then
        lastperiod := period;
        period := t;
        reinit(t, 0);
      end when;
      annotation(Documentation(info = "<html><div>
<div>Component that contains equations that count the time of one period. Also stores the last period and compares the two to calculate the time drift.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, origin = {0, -15.33}, fillColor = {0, 0, 255}, lineThickness = 2, extent = {{-56.83, -57.452}, {56.83, 57.452}}), Ellipse(visible = true, origin = {-0.165, -15.497}, fillColor = {0, 0, 255}, lineThickness = 2, extent = {{-64.788, -65.497}, {64.788, 65.497}}), Ellipse(visible = true, origin = {1.395, -15.33}, fillColor = {89, 89, 89}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-10, -10}, {10, 10}}), Polygon(visible = true, origin = {15.32, -4.757}, rotation = -53.003, fillColor = {89, 89, 89}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-10, -19.254}, {0, 38.507}, {10, -19.254}}), Rectangle(visible = true, origin = {0, 69.872}, lineColor = {89, 89, 89}, fillColor = {89, 89, 89}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-12.857, -5.031}, {12.857, 5.031}}), Rectangle(visible = true, origin = {0, 60}, lineColor = {89, 89, 89}, fillColor = {89, 89, 89}, fillPattern = FillPattern.Solid, extent = {{-5.776, -6.303}, {5.776, 6.303}}), Rectangle(visible = true, origin = {-44.532, 39.56}, rotation = 38.912, fillColor = {89, 89, 89}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-8.757, -5}, {8.757, 5}}), Rectangle(visible = true, origin = {42.843, 40}, rotation = -40.896, fillColor = {89, 89, 89}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = {{-8.757, -5}, {8.757, 5}})}), Diagram(coordinateSystem(extent = {{-50, -20}, {50, 20}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Text(visible = true, extent = {{-43.736, -16.092}, {43.736, 16.092}}, textString = "Textual model")}));
    end PeriodCounter;
    annotation(Documentation(info = "<html></html>"), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model ClockWithOneHand "Main pendulum clock example. Clock has one hand that takes one step each second. The movement is controlled by a counterweight."
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world(g = 9.81, animateWorld = false, animateGravity = false) annotation(Placement(visible = true, transformation(origin = {0, 120}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.Body pendulum(m = 1, r_CM = {0, 0, 0}, I_11 = 0, I_22 = 0, I_33 = 0, cylinderColor = {255, 215, 0}, sphereColor = {255, 215, 0}) annotation(Placement(visible = true, transformation(origin = {-60, -20}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation pendulumRope(r = {0, -0.23, 0}) annotation(Placement(visible = true, transformation(origin = {-60, 20}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Joints.Revolute revolute(phi.start = -1.13446401379631, useAxisFlange = true, phi.fixed = true, cylinderColor = {255, 215, 0}) annotation(Placement(visible = true, transformation(origin = {-60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.Fixed pendulumFixed annotation(Placement(visible = true, transformation(origin = {-60, 100}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Components.EscapementFork escapementFork annotation(Placement(visible = true, transformation(origin = {0, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    Modelica.Mechanics.Rotational.Components.Damper pendulumDampning(d = 0.01) annotation(Placement(visible = true, transformation(origin = {-20, 80}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Components.EscapementWheel escapementWheel(theta = 0.785398163397448, s0 = 0, r = 0.3) annotation(Placement(visible = true, transformation(origin = {0, 20}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.Body counterweight(m(displayUnit = "g") = 0.03, cylinderColor = {0, 230, 0}, sphereColor = {0, 230, 0}) annotation(Placement(visible = true, transformation(origin = {40, -20}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Joints.Prismatic counterweightRope(useAxisFlange = true, n = {0, -1, 0}, boxColor = {0, 230, 0}) annotation(Placement(visible = true, transformation(origin = {40, 20}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
    Modelica.Mechanics.MultiBody.Parts.Fixed counterweightFixed(r = {0, 0, -0.25}, animation = false) annotation(Placement(visible = true, transformation(origin = {40, 100}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.Rotational.Components.IdealGear secondGear(ratio = 1, useSupport = false) annotation(Placement(visible = true, transformation(origin = {0, -20}, extent = {{10, -10}, {-10, 10}}, rotation = -630)));
    Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {0, 0, 0.25}, animation = false) annotation(Placement(visible = true, transformation(origin = {-80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Joints.Revolute secondRevolute(useAxisFlange = true, n = {0, 0, -1}, phi.start = 0, phi.fixed = false, cylinderColor = {0, 0, 255}) annotation(Placement(visible = true, transformation(origin = {0, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedArrow secondHand(r_tail = {0, 0, 0}, length = 0.5, color = {0, 0, 255}) annotation(Placement(visible = true, transformation(origin = {80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Parts.FixedRotation secondAxis(rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis, angle = 90, n = {0, 0, 1}) annotation(Placement(visible = true, transformation(origin = {40, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Components.PeriodCounter periodCounter(period_0 = 0) annotation(Placement(visible = true, transformation(origin = {80, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  equation
    connect(escapementWheel.flange_b, periodCounter.flange_a) annotation(Line(visible = true, origin = {22, -12}, points = {{-22, 22}, {-22, 17}, {-2, 17}, {-2, -28}, {48, -28}}, color = {0, 127, 0}));
    connect(escapementWheel.rflange_b, secondGear.flange_a) annotation(Line(visible = true, origin = {-8, 2}, points = {{8, 18}, {-12, 18}, {-12, -2}, {8, -2}, {8, -12}}, color = {64, 64, 64}));
    connect(fixed.frame_b, secondRevolute.frame_a) annotation(Line(visible = true, origin = {-40, -60}, points = {{-30, 0}, {30, 0}}, rotation = -360, color = {95, 95, 95}));
    connect(secondGear.flange_b, secondRevolute.axis) annotation(Line(visible = true, origin = {0, -40}, points = {{0, 10}, {0, -10}}, rotation = -360, color = {64, 64, 64}));
    connect(secondRevolute.frame_b, secondAxis.frame_a) annotation(Line(visible = true, origin = {20, -60}, points = {{-10, 0}, {10, 0}}, rotation = -360, color = {95, 95, 95}));
    connect(secondAxis.frame_b, secondHand.frame_a) annotation(Line(visible = true, origin = {60, -60}, points = {{-10, 0}, {10, 0}}, rotation = -360, color = {95, 95, 95}));
    connect(counterweightFixed.frame_b, counterweightRope.frame_a) annotation(Line(visible = true, origin = {40, 60}, points = {{0, 30}, {0, -30}}, color = {95, 95, 95}));
    connect(counterweightRope.axis, escapementWheel.flange_b) annotation(Line(visible = true, origin = {17, 9.218}, points = {{17, 2.782}, {17, -4.218}, {-17, -4.218}, {-17, 0.782}}, color = {0, 127, 0}));
    connect(counterweightRope.frame_b, counterweight.frame_a) annotation(Line(visible = true, origin = {40, 0}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
    connect(escapementFork.lockFace, escapementWheel.impulseFace) annotation(Line(visible = true, origin = {-5, 40}, points = {{0, 10}, {0, -10}}, color = {0, 127, 0}));
    connect(escapementFork.impulseFace, escapementWheel.lockFace) annotation(Line(visible = true, origin = {5.423, 39.88}, points = {{-0.423, 9.88}, {-0.423, -9.88}}, color = {0, 127, 0}));
    connect(pendulumDampning.flange_a, revolute.support) annotation(Line(visible = true, origin = {-34, 84.4}, points = {{14, 5.6}, {14, 15.6}, {-6, 15.6}, {-6, -18.4}, {-16, -18.4}}, color = {64, 64, 64}));
    connect(revolute.axis, pendulumDampning.flange_b) annotation(Line(visible = true, origin = {-30, 63.333}, points = {{-20, -3.333}, {10, -3.333}, {10, 6.667}}, color = {64, 64, 64}));
    connect(revolute.axis, escapementFork.flange_a) annotation(Line(visible = true, origin = {-25, 60}, points = {{-25, 0}, {25, 0}}, color = {64, 64, 64}));
    connect(pendulum.frame_a, pendulumRope.frame_b) annotation(Line(visible = true, origin = {-60, 0}, points = {{0, -10}, {0, 10}}, color = {95, 95, 95}));
    connect(revolute.frame_b, pendulumRope.frame_a) annotation(Line(visible = true, origin = {-60, 40}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
    connect(pendulumFixed.frame_b, revolute.frame_a) annotation(Line(visible = true, origin = {-60, 80}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
    annotation(__Wolfram, experiment(StopTime = 60, __Wolfram_SynchronizeWithRealTime = false), Documentation(info = "<html>
<p>This model shows the mechanics of a pendulum clock.</p>

<h4>Dynamics</h4>

<p>A pendulum clock works by having a pendulum swinging back and forth with a constant period of one second. The pendulum period can be directly determined by the length of the pendulum rod and the gravity affecting the pendulum. The swinging motion pushes on a fork (see graphic), which releases an escapement wheel that is attached to a weight. When the gear is released, gravity pulls the weight down and the gear starts to turn. When the pendulum swings back, the gear is locked once again. The angled cogs and the torque applied to the escapement wheel give the pendulum a small push that returns energy to the pendulum that was lost due to friction.</p>

<p style=\"text-align: center;\"><img src=\"modelica://PendulumClock/DocumentationFiles/anchor.png\" style=\"width:149.5px\" alt=\"anchor\" /></p>



<h4>Plot the Results</h4>

<p>Click the Simulate button: <img src=\"modelica://PendulumClock/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simulate\" /></p>

<p>Explore how the hand is rotated during one minute. The secondGear.phi_b variable describes the change in degrees, counting from the start of the simulation. After 60 seconds of simulation, the hand will have rotated 360 degrees.</p>

<p>A plot will automatically be displayed when the simulation has finished. You should see the plot below:</p>


<p style=\"text-align: center;\"><img src=\"modelica://PendulumClock/DocumentationFiles/plot.png\" style=\"width:600px\" alt=\"\" /></p>

<h4>Automatic 3D Animation</h4>

<p>Multibody systems have visualizers to show what a real-world system would look like.</p>

<p style=\"text-align: center;\"><img src=\"modelica://PendulumClock/DocumentationFiles/sim.png\" style=\"width:600px\" alt=\"sim\" /></p>

<p>To show a 3D animation of the model, follow the steps below:</p>

<p>Click the Simulate button: <img src=\"modelica://PendulumClock/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"\" /></p>

<p>Click the Animation button: <img src=\"modelica://PendulumClock/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\" /></p>
<p>Use your mouse or trackpad to drag the animation to a good angle and zoom in with your scroll wheel or by using the trackpad. Then click the Play button: <img src=\"modelica://PendulumClock/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"play\" /></p>



<h4>Expand the Model</h4>
 
<p>Using System Modeler's drag-and-drop functionality, you can expand the model to include a minute hand. First you need to create a new hand. This is done by duplicating the second hand:</p>

<ul>
<li>Select the five components shown in the graphic, either by applying Shift+Click to each one individually or selecting them using click-and-drag.</li>
</ul>


<p style=\"text-align: center;\"><img src=\"modelica://PendulumClock/DocumentationFiles/clickanddrag.png\" style=\"width:600px\" alt=\"clickanddrag\" /></p>

<ul>
<li>Press Ctrl+D to duplicate the components.</li>
<li>Drag the new components toward the bottom of the diagram layer and release them by clicking anywhere on the diagram layer.</li>
</ul>

<p>Next you need to make the connections to the new hand:</p>



<p>Select the Connection Line Tool button <img src=\"modelica://PendulumClock/DocumentationFiles/connection_line_tool.svg\" style=\"width:20px\" alt=\"connection\" /> from the toolbar or press C.</p>
<p>Click the white circle (flange_b) and then click the gray circle on the duplicated gear (flange_a). You can click anywhere in the diagram layer to draw your connection line through that point. Try clicking around the first gear component to get a clean model diagram.</p>


<p>Your diagram should now look something like this:</p>

<p style=\"text-align: center;\"><img src=\"modelica://PendulumClock/DocumentationFiles/duplicated.png\" style=\"width:600px\"  alt=\"duplicated\" /></p>

<p>Now the new hand can be made distinct from the old one:</p>

<ul>
<li>Click the duplicated gear. You want the minute hand to turn 1/60 degrees every time the second hand turns one degree. In the General tab, next to the ratio parameter, write 60.</li>
<li>The duplicated components are now named secondGear1, secondRevolute1, etc. If you want, you can rename them to something more appropriate, like minuteGear, minuteRevolute, and so on. Select each of the duplicated components and press F2. A Component Properties window will show up; in the box next to \"Name:\", enter your desired component name and click OK.</li>

</ul>

<p>Repeat the steps in the Simulation section to see the effects of your changes.</p>
</html>", figures = {Figure(title = "Second hand angle", identifier = "1b9cf", preferred = true, plots = {Plot(curves = {Curve(y = secondGear.phi_b, legend = "Second hand angle over time")}, y = Axis(label = "Angle from top position")), Plot(curves = {Curve(y = secondHand.frame_a.R.w[3], legend = "Second hand velocity over time")}, y = Axis(label = "Angular velocity"))})}), preferedView = "info", Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Diagram(coordinateSystem(extent = {{-110, -95}, {100, 150}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end ClockWithOneHand;

  model ClockWithTwoHands "Expanded pendulum clock setup with two hands, the relative motion of the hands are controlled by gear ratios."
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world annotation(Placement(visible = true, transformation(origin = {0, 120}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.Body pendulum(m = 1) annotation(Placement(visible = true, transformation(origin = {-60, -20}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation pendulumRope(r = {0, -0.25, 0}) annotation(Placement(visible = true, transformation(origin = {-60, 20}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Joints.Revolute revolute(phi.start = -1.13446401379631, useAxisFlange = true, phi.fixed = true) annotation(Placement(visible = true, transformation(origin = {-60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.Fixed pendulumFixed annotation(Placement(visible = true, transformation(origin = {-60, 100}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    PendulumClock.Components.EscapementFork escapementFork annotation(Placement(visible = true, transformation(origin = {0, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    Modelica.Mechanics.Rotational.Components.Damper pendulumDampning(d = 0.01) annotation(Placement(visible = true, transformation(origin = {-20, 80}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    PendulumClock.Components.EscapementWheel escapementWheel(theta = 0.785398163397448, r = 0.3) annotation(Placement(visible = true, transformation(origin = {0, 20}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.Body counterweight(m(displayUnit = "g") = 0.03) annotation(Placement(visible = true, transformation(origin = {40, -20}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Joints.Prismatic counterweightRope(useAxisFlange = true, n = {0, -1, 0}) annotation(Placement(visible = true, transformation(origin = {40, 20}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
    Modelica.Mechanics.MultiBody.Parts.Fixed counterweightFixed annotation(Placement(visible = true, transformation(origin = {40, 100}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.Rotational.Components.IdealGear secondGear(ratio = 1, useSupport = false) annotation(Placement(visible = true, transformation(origin = {0, -20}, extent = {{10, -10}, {-10, 10}}, rotation = -630)));
    Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {0, 0, 0.25}, animation = false) annotation(Placement(visible = true, transformation(origin = {-80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Joints.Revolute secondRevolute(useAxisFlange = true, n = {0, 0, -1}, phi.start = 0, phi.fixed = false, cylinderColor = {0, 0, 255}) annotation(Placement(visible = true, transformation(origin = {0, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedArrow secondHand(r_tail = {0, 0, 0}, length = 0.5, color = {0, 0, 255}) annotation(Placement(visible = true, transformation(origin = {80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Parts.FixedRotation secondAxis(rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis, angle = 90, n = {0, 0, 1}) annotation(Placement(visible = true, transformation(origin = {40, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    PendulumClock.Components.PeriodCounter periodCounter(period_0 = 0.5) annotation(Placement(visible = true, transformation(origin = {80, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.Rotational.Components.IdealGear minuteGear(ratio = 60, useSupport = false) annotation(Placement(visible = true, transformation(origin = {0, -100}, extent = {{10, -10}, {-10, 10}}, rotation = -630)));
    Modelica.Mechanics.MultiBody.Parts.Fixed fixed1(r = {0, 0, 0.25}, animation = false) annotation(Placement(visible = true, transformation(origin = {-80, -140}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Joints.Revolute minuteRevolute(useAxisFlange = true, n = {0, 0, -1}, phi.start = 0, phi.fixed = false, cylinderColor = {0, 0, 255}) annotation(Placement(visible = true, transformation(origin = {0, -140}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedArrow minuteHand(r_tail = {0, 0, 0}, length = 0.5, color = {0, 0, 255}) annotation(Placement(visible = true, transformation(origin = {80, -140}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Parts.FixedRotation minuteAxis(rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis, angle = 90, n = {0, 0, 1}) annotation(Placement(visible = true, transformation(origin = {40, -140}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
  equation
    connect(minuteAxis.frame_b, minuteHand.frame_a) annotation(Line(visible = true, origin = {60, -140}, points = {{-10, 0}, {10, 0}}, color = {95, 95, 95}));
    connect(minuteRevolute.frame_b, minuteAxis.frame_a) annotation(Line(visible = true, origin = {20, -140}, points = {{-10, 0}, {10, 0}}, color = {95, 95, 95}));
    connect(secondGear.flange_b, minuteGear.flange_a) annotation(Line(visible = true, origin = {-6.667, -60}, points = {{6.667, 30}, {6.667, 20}, {-13.333, 20}, {-13.333, -20}, {6.667, -20}, {6.667, -30}}, color = {64, 64, 64}));
    connect(minuteGear.flange_b, minuteRevolute.axis) annotation(Line(visible = true, origin = {0, -120}, points = {{0, 10}, {0, -10}}, rotation = -360, color = {64, 64, 64}));
    connect(fixed1.frame_b, minuteRevolute.frame_a) annotation(Line(visible = true, origin = {-40, -140}, points = {{-30, 0}, {30, 0}}, rotation = -360, color = {95, 95, 95}));
    connect(escapementWheel.flange_b, periodCounter.flange_a) annotation(Line(visible = true, origin = {22, -12}, points = {{-22, 22}, {-22, 17}, {-2, 17}, {-2, -28}, {48, -28}}, color = {0, 127, 0}));
    connect(escapementWheel.rflange_b, secondGear.flange_a) annotation(Line(visible = true, origin = {-8, 2}, points = {{8, 18}, {-12, 18}, {-12, -2}, {8, -2}, {8, -12}}, color = {64, 64, 64}));
    connect(fixed.frame_b, secondRevolute.frame_a) annotation(Line(visible = true, origin = {-40, -60}, points = {{-30, 0}, {30, 0}}, rotation = -360, color = {95, 95, 95}));
    connect(secondGear.flange_b, secondRevolute.axis) annotation(Line(visible = true, origin = {0, -40}, points = {{0, 10}, {0, -10}}, rotation = -360, color = {64, 64, 64}));
    connect(secondRevolute.frame_b, secondAxis.frame_a) annotation(Line(visible = true, origin = {20, -60}, points = {{-10, 0}, {10, 0}}, rotation = -360, color = {95, 95, 95}));
    connect(secondAxis.frame_b, secondHand.frame_a) annotation(Line(visible = true, origin = {60, -60}, points = {{-10, 0}, {10, 0}}, rotation = -360, color = {95, 95, 95}));
    connect(counterweightFixed.frame_b, counterweightRope.frame_a) annotation(Line(visible = true, origin = {40, 60}, points = {{0, 30}, {0, -30}}, color = {95, 95, 95}));
    connect(counterweightRope.axis, escapementWheel.flange_b) annotation(Line(visible = true, origin = {17, 9.218}, points = {{17, 2.782}, {17, -4.218}, {-17, -4.218}, {-17, 0.782}}, color = {0, 127, 0}));
    connect(counterweightRope.frame_b, counterweight.frame_a) annotation(Line(visible = true, origin = {40, 0}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
    connect(escapementFork.lockFace, escapementWheel.impulseFace) annotation(Line(visible = true, origin = {-5, 40}, points = {{0, 10}, {0, -10}}, color = {0, 127, 0}));
    connect(escapementFork.impulseFace, escapementWheel.lockFace) annotation(Line(visible = true, origin = {5, 39.88}, points = {{0, 9.88}, {0, -9.88}}, color = {0, 127, 0}));
    connect(pendulumDampning.flange_a, revolute.support) annotation(Line(visible = true, origin = {-34, 84.4}, points = {{14, 5.6}, {14, 15.6}, {-6, 15.6}, {-6, -18.4}, {-16, -18.4}}, color = {64, 64, 64}));
    connect(revolute.axis, pendulumDampning.flange_b) annotation(Line(visible = true, origin = {-30, 63.333}, points = {{-20, -3.333}, {10, -3.333}, {10, 6.667}}, color = {64, 64, 64}));
    connect(revolute.axis, escapementFork.flange_a) annotation(Line(visible = true, origin = {-25, 60}, points = {{-25, 0}, {25, 0}}, color = {64, 64, 64}));
    connect(pendulum.frame_a, pendulumRope.frame_b) annotation(Line(visible = true, origin = {-60, 0}, points = {{0, -10}, {0, 10}}, color = {95, 95, 95}));
    connect(revolute.frame_b, pendulumRope.frame_a) annotation(Line(visible = true, origin = {-60, 40}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
    connect(pendulumFixed.frame_b, revolute.frame_a) annotation(Line(visible = true, origin = {-60, 80}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
    annotation(__Wolfram, experiment(StopTime = 60, __Wolfram_SynchronizeWithRealTime = false), Documentation(info = "<html><div>
<div><span>Expanded model from the </span><a href=\"modelica://PendulumClock.ClockWithOneHand\">example model</a><span>. Clock with two hands instead of one. Follow the steps in the example model to create this model.</span></div>
</div></html>", figures = {Figure(title = "Minute hand angle", identifier = "8e613", preferred = true, plots = {Plot(curves = {Curve(y = minuteGear.phi_b, legend = "Minute hand angle over time")}, y = Axis(label = "Angle from top position")), Plot(curves = {Curve(y = minuteHand.frame_a.R.w[3], legend = "Minute hand velocity over time")}, y = Axis(label = "Angular velocity"))}), Figure(title = "Second hand angle", identifier = "014a5", plots = {Plot(curves = {Curve(y = secondGear.phi_b, legend = "Second hand angle over time")}, y = Axis(label = "Angle from top position")), Plot(curves = {Curve(y = secondHand.frame_a.R.w[3], legend = "Second hand velocity over time")}, y = Axis(label = "Angular velocity"))})}), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Diagram(coordinateSystem(extent = {{-148.5, -170}, {148.5, 150}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end ClockWithTwoHands;
  annotation(Documentation(info = "<html></html>"), uses(Modelica(version = "4.0.0")), __Wolfram(openClass = PendulumClock.ClockWithOneHand), preferredView = "info", Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
end PendulumClock;
